/*
  docgen  Objective C Document Generator
  Copyright (C) 1995  Bill Bereza.
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
  Email:
  berezaw@river.it.gvsu.edu

  S-mail:
  Bill Bereza
  9526 Judson Rd.
  Ravenna, MI 49451-9427

  */
/* docgen	Objective C Document Generator
 * Copyright (c) 1994 Bill Bereza
 *
 * $Log:	Formatting.h,v $
 * Revision 1.4  95/07/26  21:46:34  berezaw
 * changed definition of formatExtension
 * 
 * Revision 1.3  95/07/24  20:06:49  berezaw
 * *** empty log message ***
 * 
 * Revision 1.2  95/07/21  21:40:31  berezaw
 * *** empty log message ***
 * 
 * Revision 1.1  95/07/21  01:59:10  berezaw
 * Initial revision
 * 
 *
 *	$Id: Formatting.h,v 1.4 95/07/26 21:46:34 berezaw Exp $
 */

#ifndef _FORMATTING_H
#define _FORMATTING_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <objc/Object.h>

#ifdef STDC_HEADERS
#include <stdio.h>
#endif

@protocol Formatting
/*
   Formatting is a protocol for producing fancy or un-fancy text output.

   All the methods should return what is returned by the writeFormat: method.
   */

// This method should be used to write
// If I had libobjects on this NeXT, I would use the StdioStream class
// for this to make it less dependent
//

- (int) writeFormat: (const char *)format, ...;

// Here's some code for this if you don't want to subclass
// the DocgenFormatter class:

/* Taken from StdioStream.m from libobjects 0.1.3
   Copyright (C) 1994 Free Software Foundation, Inc.

   Written by:  R. Andrew McCallum <mccallum@gnu.ai.mit.edu>
   Date: July 1994
*/
/*
- (int) writeFormat: (const char *)format, ...
{
  int ret;
  va_list ap;

  va_start(ap, format);
  ret = vfprintf(fp, format, ap);
  va_end(ap);
  return ret;
}
*/

// Generic formatting commands
-(int)changeFontSize:(int)to;
-(int)putLine:(const char *)line;
-(int)makeBold;
-(int)unBold;
-(int)makeItalic;
-(int)unItalic;
-(int)changeFont:(int)to;
-(int)makeUnderline;
-(int)unUnderline;

@end

@protocol DocgenFormatting <Formatting>
/*
   DocgenFormatting adds Docgen specific formatting to Formatting.

   *A* *Note:* You should %not% close the file yourself. That should be done by whoever fopen'd the file!
   */

+(char *)fileExtension;

-initWithOfile:(FILE *)of;

-setOfile:(FILE *)of;
-(FILE *)Ofile;

-(int)putEof;
-(int)putConformHeader:(const char *)line;
-(int)putMethodL1;
-(int)putMethodL2;
-(int)putHead;
-(int)putCPHead;
-(int)putMethodDescription;
-(int)putFont:(int)fn Char:(int)theChar;
-(int)putStart;
-(int)putInheritHeader;
-(int)putInheritTail:(const char *)line;
-(int)putCategoryHeader;
-(int)putCategoryTail:(const char *)line;
-(int)putIHeader;
-(int)putMethodHeader:(const char *)kindof;
-(int)putMethodType;
-(int)putIVarDescription;

@end

#endif /* _FORMATTING_H */
